@@ -31,7 +31,7 @@ import datetime
 import time
 import collections
 
-VERSION = (0, 3, 6)
+VERSION = (0, 3, 7)
 __version__ = '%s.%s.%s' % VERSION
 apilevel = '2.0'
 threadsafety = 1
@@ -175,7 +175,7 @@ PG_TYPE_FDW_HANDLER = 3115
 PG_TYPE_ANYRANGE = 3831
 
 class TZ(datetime.tzinfo):
-    def __init__(self, offset):
+    def __init__(self, offset='+00'):
         self.offset = offset
         hours = int(self.offset[:3])
         if len(offset) > 3:
@@ -400,13 +400,15 @@ class Cursor(object):
             raise ProgrammingError(u"08003:Lost connection")
         self.description = []
         self._rows.clear()
-        self.query = query
         self.args = args
         if args:
-            escaped_args = tuple(self.connection.escape_parameter(arg) for arg in args)
-            query = query.replace('%', '%%').replace('%%s', '%s')
+            escaped_args = tuple(
+                self.connection.escape_parameter(arg).replace(u'%', u'%%') for arg in args
+            )
+            query = query.replace(u'%', u'%%').replace(u'%%s', u'%s')
             query = query % escaped_args
-            query = query.replace('%%', '%')
+            query = query.replace(u'%%', u'%')
+        self.query = query
         self.connection.execute(query, self)
 
     def executemany(self, query, seq_of_params):
@@ -546,19 +548,25 @@ class Connection(object):
                     for i in range(count):
                         name = data[n:n+data[n:].find(b'\x00')]
                         n += len(name) + 1
+                        type_code = _bytes_to_bint(data[n+6:n+10])
+                        if type_code == PG_TYPE_VARCHAR:
+                            size = _bytes_to_bint(data[n+12:n+16]) - 4
+                            precision = None
+                            scale = None
+                        elif type_code == PG_TYPE_NUMERIC:
+                            size = _bytes_to_bint(data[n+10:n+12])
+                            precision = _bytes_to_bint(data[n+12:n+14])
+                            scale = precision - _bytes_to_bint(data[n+14:n+16])
+                        else:
+                            size = _bytes_to_bint(data[n+10:n+12])
+                            precision =None
+                            scale = None
 #                        table_oid = _bytes_to_bint(data[n:n+4])
 #                        table_pos = _bytes_to_bint(data[n+4:n+6])
+#                        size = _bytes_to_bint(data[n+10:n+12])
 #                        modifier = _bytes_to_bint(data[n+12:n+16])
 #                        format = _bytes_to_bint(data[n+16:n+18]),
-                        field = (
-                            name,
-                            _bytes_to_bint(data[n+6:n+10]),     # type oid
-                            None,
-                            _bytes_to_bint(data[n+10:n+12]),    # size
-                            None,
-                            None,
-                            None,
-                        )
+                        field = (name, type_code, None, size, precision, scale, None)
                         n += 18
                         obj.description[idx] = field
                         idx += 1
@@ -621,6 +629,11 @@ class Connection(object):
                 if DEBUG: DEBUG_OUTPUT("SKIP:%d\t%i\t%s" % (code, ln, HEX(data)))
         return errobj
 
+    def process_messages(self, obj):
+        err = self._process_messages(obj)
+        if err:
+            raise err
+
     def _read(self, ln):
         if not self.sock:
             raise OperationalError(u"08003:Lost connection")
@@ -664,9 +677,7 @@ class Connection(object):
         v += b'\x00'
 
         self._write(_bint_to_bytes(len(v) + 4) + v)
-        err = self._process_messages(None)
-        if err:
-            raise err
+        self.process_messages(None)
 
     def escape_parameter(self, v):
         t = type(v)
@@ -721,9 +732,7 @@ class Connection(object):
             PG_F_QUERY,
             query.encode(self.encoding) + b'\x00',
         )
-        err = self._process_messages(obj)
-        if err:
-            raise err
+        self.process_messages(obj)
         if self.autocommit:
             self.commit()
 
@@ -746,19 +755,13 @@ class Connection(object):
         if DEBUG: DEBUG_OUTPUT('COMMIT')
         if self.sock:
             self._send_message(PG_F_QUERY, b"COMMIT\x00")
-            self._process_messages(None)
+            self.process_messages(None)
 
-    def _rollback(self):
+    def rollback(self):
         if self.sock:
             if DEBUG: DEBUG_OUTPUT('ROLLBACK')
             self._send_message(PG_F_QUERY, b"ROLLBACK\x00")
-            return self._process_messages(None)
-        return None
-
-    def rollback(self):
-        err = self._rollback()
-        if err:
-            raise err
+        self.process_messages(None)
 
     def reopen(self):
         self.close()
